@@ -9,9 +9,9 @@ import types
 from functools import partial
 
 from .cli import emph, parse_args
-from .constants import appname, version
-from .utils import parse_address_spec, read_with_timeout
 from .cmds import cmap, parse_subcommand_cli
+from .constants import appname, version
+from .utils import TTYIO, parse_address_spec
 
 
 def handle_cmd(boss, window, cmd):
@@ -33,7 +33,7 @@ def handle_cmd(boss, window, cmd):
 global_options_spec = partial('''\
 --to
 An address for the kitty instance to control. Corresponds to the address
-given to the kitty instance via the --listen-on option. If not specified,
+given to the kitty instance via the :option:`kitty --listen-on` option. If not specified,
 messages are sent to the controlling terminal for this process, i.e. they
 will only work if this process is run within an existing kitty window.
 '''.format, appname=appname)
@@ -44,39 +44,74 @@ def encode_send(send):
     return b'\x1bP' + send + b'\x1b\\'
 
 
+class SocketIO:
+
+    def __init__(self, to):
+        self.family, self.address = parse_address_spec(to)[:2]
+
+    def __enter__(self):
+        import socket
+        self.socket = socket.socket(self.family)
+        self.socket.setblocking(True)
+        self.socket.connect(self.address)
+
+    def __exit__(self, *a):
+        import socket
+        self.socket.shutdown(socket.SHUT_RDWR)
+        self.socket.close()
+
+    def send(self, data):
+        import socket
+        with self.socket.makefile('wb') as out:
+            if isinstance(data, bytes):
+                out.write(data)
+            else:
+                for chunk in data:
+                    if isinstance(chunk, str):
+                        chunk = chunk.encode('utf-8')
+                    out.write(chunk)
+                    out.flush()
+        self.socket.shutdown(socket.SHUT_WR)
+
+    def recv(self, more_needed, timeout):
+        # We dont bother with more_needed since the server will close the
+        # connection after transmission
+        self.socket.settimeout(timeout)
+        with self.socket.makefile('rb') as src:
+            data = src.read()
+        more_needed(data)
+
+
 def do_io(to, send, no_response):
-    import socket
-    send = encode_send(send)
-    if to:
-        family, address = parse_address_spec(to)[:2]
-        s = socket.socket(family)
-        s.connect(address)
-        out = s.makefile('wb')
+    payload = send.get('payload')
+    if not isinstance(payload, types.GeneratorType):
+        send_data = encode_send(send)
     else:
-        out = open('/dev/tty', 'wb')
-    with out:
-        out.write(send)
-    if to:
-        s.shutdown(socket.SHUT_WR)
-    if no_response:
-        return {'ok': True}
+        def send_generator():
+            for chunk in payload:
+                send['payload'] = chunk
+                yield encode_send(send)
+        send_data = send_generator()
 
-    received = b''
-    dcs = re.compile(br'\x1bP@kitty-cmd([^\x1b]+)\x1b\\')
-    match = None
+    io = SocketIO(to) if to else TTYIO()
+    with io:
+        io.send(send_data)
+        if no_response:
+            return {'ok': True}
 
-    def more_needed(data):
-        nonlocal received, match
-        received += data
-        match = dcs.search(received)
-        return match is None
+        dcs = re.compile(br'\x1bP@kitty-cmd([^\x1b]+)\x1b\\')
+
+        received = b''
+        match = None
+
+        def more_needed(data):
+            nonlocal received, match
+            received += data
+            match = dcs.search(received)
+            return match is None
+
+        io.recv(more_needed, timeout=10)
 
-    if to:
-        src = s.makefile('rb')
-    else:
-        src = open('/dev/tty', 'rb')
-    with src:
-        read_with_timeout(more_needed, src=src)
     if match is None:
         raise SystemExit('Failed to receive response from ' + appname)
     response = json.loads(match.group(1).decode('ascii'))
@@ -84,18 +119,19 @@ def do_io(to, send, no_response):
 
 
 all_commands = tuple(sorted(cmap))
+cli_msg = (
+        'Control {appname} by sending it commands. Set the'
+        ' :opt:`allow_remote_control` option to yes in :file:`kitty.conf` for this'
+        ' to work.'
+).format(appname=appname)
 
 
 def parse_rc_args(args):
-    cmds = ('  |G {}|\n    {}'.format(cmap[c].name, cmap[c].short_desc) for c in all_commands)
-    msg = (
-        'Control {appname} by sending it commands. Add'
-        ' |_ allow_remote_control yes| to kitty.conf for this'
-        ' to work.\n\n|T Commands|:\n{cmds}\n\n'
-        'You can get help for each individual command by using:\n'
-        '{appname} @ |_ command| -h'
-    ).format(appname=appname, cmds='\n'.join(cmds))
-
+    cmds = ('  :green:`{}`\n    {}'.format(cmap[c].name, cmap[c].short_desc) for c in all_commands)
+    msg = cli_msg + (
+            '\n\n:title:`Commands`:\n{cmds}\n\n'
+            'You can get help for each individual command by using:\n'
+            '{appname} @ :italic:`command` -h').format(appname=appname, cmds='\n'.join(cmds))
     return parse_args(args[1:], global_options_spec, 'command ...', msg, '{} @'.format(appname))
 
 
@@ -118,11 +154,6 @@ def main(args):
         'cmd': cmd,
         'version': version,
     }
-    if func.no_response and isinstance(payload, types.GeneratorType):
-        for item in payload:
-            send['payload'] = item
-            do_io(global_opts.to, send, func.no_response)
-        return
     if payload is not None:
         send['payload'] = payload
     response = do_io(global_opts.to, send, func.no_response)
